

Application create: #NetEnablingPackage with:
    (#( EtTools)
        collect: [:each | Smalltalk at: each ifAbsent: [
            Application errorPrerequisite: #NetEnablingPackage missing: each]])!

NetEnablingPackage becomeDefault!
Object subclass: #DirectPlay
    instanceVariableNames: 'logonType '
    classVariableNames: ''
    poolDictionaries: ''!

NetEnablingPackage becomeDefault!
EtClassesBrowser subclass: #NetEnabledBrowser
    instanceVariableNames: 'directPlayObject pollingProcess '
    classVariableNames: 'ActiveNetEnabledBrowser '
    poolDictionaries: ''!

NetEnablingPackage becomeDefault!
Application subclass: #NetEnablingPackage
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!


NetEnablingPackage becomeDefault!

!DirectPlay class publicMethods !

timeoutInMilliseconds
	^5000! !

DirectPlay class categoriesFor: #timeoutInMilliseconds are:((OrderedCollection new)
	add: 'timing control';
	yourself)!

!DirectPlay publicMethods !

asString: pointer
	"Converts a pointer to a string from the DLL into a Smalltalk string."
	^(OSStringZ address: pointer) asString!

asStrings: aStringOrByteArray
	"Converts a string into a collection of strings."
	aStringOrByteArray isEmpty 
		ifTrue: [^#()]
		ifFalse: [^aStringOrByteArray asString subStrings: self stringSeparator]!

chooseNetworkProvider: index
	^self privateChooseProvider: index!

choosePlayer: index
	^self privateChoosePlayer: index!

chooseSession: index
	^self privateChooseSession: index!

close
	^self privateCloseDirectPlay!

errorString
	^self asString: self privateErrorString!

logonAsClientNamed:  formalName and: friendlyName
	logonType := #client.
	self 
		privateClient: formalName nullTerminated 
		and: friendlyName nullTerminated!

logonAsServerNamed:  formalName and: friendlyName
	logonType := #server.
	self 
		privateServer: formalName nullTerminated
		and: friendlyName nullTerminated!

logonType
	"Returns #server or #client"
	^logonType!

networkProviders
	"Providers are collections of strings."
	^self asStrings: (self asString: self privateProviderList)!

networkSessions
	"Sessions are collections of strings."
	^self asStrings: (self asString: (self privateSessionList: self class timeoutInMilliseconds)) !

open: guidString
	self privateOpenDirectPlay: guidString nullTerminated!

packIntoByteArray: OSObjectOrByteArray
	"Packed format: length (4 bytes), actual data (length bytes)."
	| size bytes |
	size := OSObjectOrByteArray size. 
	bytes := ByteArray new: size + 4.
	bytes int32At: 0 put: size.
	OSObjectOrByteArray memcpyFrom: 0 to: size - 1 into: bytes startingAt: 4.
	^bytes!

players
	"Players are collections of 'formal name, friendly name' strings."
	| strings pairs |
	strings := self asStrings: (self asString: self privatePlayerList).
	pairs := OrderedCollection new.
	1 to: strings size // 2 by: 2 do: [:index |
		pairs add: (strings at: index), ', ', (strings at: index+1)].
	^pairs!

privateChoosePlayer: index
	<c: void 'netEnabler' :choosePlayer int32>
	^self primitiveFailed!

privateChooseProvider: index
	<c: void 'netEnabler' :chooseProvider int32>
	^self primitiveFailed!

privateChooseSession: index
	<c: void 'netEnabler' :chooseSession int32>
	^self primitiveFailed!

privateClient: formalName and: friendlyName
	<c: void 'netEnabler' :client struct struct>
	^self primitiveFailed!

privateCloseDirectPlay
	<c: void 'netEnabler' :closeDirectPlay>
	^self primitiveFailed!

privateErrorString
	<c: pointer 'netEnabler' :lastError>
	^self primitiveFailed!

privateOpenDirectPlay: guidDoubleByteString
	<c: void 'NetEnabler' :openDirectPlay struct>
	^self primitiveFailed


!

privatePlayerList
	<c: pointer 'netEnabler' :playerList>
	^self primitiveFailed!

privateProviderList
	<c: pointer 'netEnabler' :providerList>
	^self primitiveFailed!

privateReceive
	"Receives a pointer to a byte array encoded as follows: 
	size (4 bytes), other data (rest of byteArray)."
	<c: pointer 'netEnabler' :receive>
	^self primitiveFailed!

privateSend: byteArray
	"Sends a byte array encoded as follows: 
	size (4 bytes), other data (rest of byteArray)."
	<c: void 'netEnabler' :send pointer>
	^self primitiveFailed!

privateServer: formalName and: friendlyName
	<c: void 'netEnabler' :server struct struct>
	^self primitiveFailed!

privateSessionList: timeoutInMilliseconds
	<c: pointer 'netEnabler' :sessionList int32>
	^self primitiveFailed!

receiveAs: OSclass
	"Receive into a fixed or variable size structure."
	| pointer |
	pointer := self privateReceive.
	OSclass variableSize = 0
		ifTrue: [^self unpack: pointer intoFixedStructure: OSclass]
		ifFalse: [^self unpack: pointer intoVariableStructure: OSclass]!

receiveByteArray
	^(self receiveAs: OSByteArray) asBytes!

receiveStrings
	^self asStrings: self receiveByteArray!

sendAs: anOSObject
	| bytes |
	bytes := self packIntoByteArray: anOSObject
	self privateSend: bytes!

sendByteArray: aByteArray
	self sendAs: aByteArray!

sendStrings: aCollection
	"Packed format: length (4 bytes), actual data (length bytes)."
	| stream bytes |
	stream := WriteStream on: (String new: 400).
	stream nextPutAll: '    '. "room for length"
	aCollection do: [:string | 
		stream nextPutAll: string; nextPut: self stringSeparator].
	stream skip: -1.
	bytes := stream contents asByteArray.	
	bytes int32At: 0 put: bytes size - 4.
	self privateSend: bytes!

stringSeparator
	^1 asCharacter!

unpack: pointer intoFixedStructure: OSclass
	"Unpack into a fixed size structure. 
	Packed format: length (4 bytes), actual data (length bytes)."
	| actualByteSize requiredByteSize |
	actualByteSize := (OSObject address: pointer) int32At: 0.
	requiredByteSize :=  OSclass fixedSize.
	self validateFixedSizes: actualByteSize and: requiredByteSize.
	"NB: Return a copy since direct play reuses its message buffers."
	^(OSclass address: pointer offset: 4) copy!

unpack: pointer intoVariableStructure: OSclass
	"Unpack into a variable size structure. 
	Packed format: length (4 bytes), actual data (length bytes)."
	| structure actualByteSize requiredByteSize count |
	actualByteSize := (OSObject address: pointer) int32At: 0.
	count := (actualByteSize - OSclass fixedSize) // OSclass variableSize.
	requiredByteSize := OSclass fixedSize + (OSclass variableSize * count).
	self validateVariableSizes: actualByteSize and: requiredByteSize.
	structure := OSclass newVariable: count.
	(OSObject address: pointer offset: 4)
		memcpyFrom: 0 to: actualByteSize - 1 into: structure startingAt: 0.
	^structure!

validateFixedSizes: actualSize and: requiredSize
	actualSize = requiredSize ifTrue: [^self].
	^self error: 'Received ', 
		actualSize printString, 
		' bytes for conversion to ', 
		requiredSize printString, 
		' byte OS structure'
!

validateVariableSizes: actualSize and: requiredSize
	actualSize = requiredSize ifTrue: [^self].
	^self error: 'Received ', 
		actualSize printString, 
		' bytes for conversion to OS structure; ',
		(requiredSize - actualSize) printString, 
		' excess bytes dropped.'! !

DirectPlay categoriesFor: #asStrings: are:((OrderedCollection new)
	add: 'utilities';
	yourself)!
DirectPlay categoriesFor: #chooseNetworkProvider: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #privateSend: are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #privateSessionList: are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #privateChoosePlayer: are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #chooseSession: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #unpack:intoFixedStructure: are:((OrderedCollection new)
	add: 'interacting support';
	yourself)!
DirectPlay categoriesFor: #privateChooseSession: are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #privateErrorString are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #privatePlayerList are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #privateChooseProvider: are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #players are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #unpack:intoVariableStructure: are:((OrderedCollection new)
	add: 'interacting support';
	yourself)!
DirectPlay categoriesFor: #open: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #validateVariableSizes:and: are:((OrderedCollection new)
	add: 'interacting support';
	yourself)!
DirectPlay categoriesFor: #privateReceive are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #receiveAs: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #sendStrings: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #privateProviderList are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #packIntoByteArray: are:((OrderedCollection new)
	add: 'interacting support';
	yourself)!
DirectPlay categoriesFor: #errorString are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #asString: are:((OrderedCollection new)
	add: 'utilities';
	yourself)!
DirectPlay categoriesFor: #privateOpenDirectPlay: are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #privateClient:and: are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #stringSeparator are:((OrderedCollection new)
	add: 'utilities';
	yourself)!
DirectPlay categoriesFor: #privateCloseDirectPlay are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #logonAsServerNamed:and: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #receiveByteArray are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #receiveStrings are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #networkProviders are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #logonType are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #logonAsClientNamed:and: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #privateServer:and: are:((OrderedCollection new)
	add: 'relaying to the DLL';
	yourself)!
DirectPlay categoriesFor: #sendByteArray: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #close are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #sendAs: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #networkSessions are:((OrderedCollection new)
	add: 'interacting';
	yourself)!
DirectPlay categoriesFor: #validateFixedSizes:and: are:((OrderedCollection new)
	add: 'interacting support';
	yourself)!
DirectPlay categoriesFor: #choosePlayer: are:((OrderedCollection new)
	add: 'interacting';
	yourself)!

!NetEnabledBrowser class publicMethods !

pollingRateInMilliseconds
	^2000!

reset
	ActiveNetEnabledBrowser := nil! !

NetEnabledBrowser class categoriesFor: #reset are:((OrderedCollection new)
	add: 'when under duress';
	yourself)!
NetEnabledBrowser class categoriesFor: #pollingRateInMilliseconds are:((OrderedCollection new)
	add: 'timing control';
	yourself)!

!NetEnabledBrowser publicMethods !

booleanPrompt: title
	^System confirm: title!

changedSelectedMethods
	super changedSelectedMethods.
	self informDirectPlayOfSelection!

close
	self disconnectFromNetwork.
	^super close!

closeDirectPlay
	directPlayObject isNil ifFalse: [directPlayObject close].
	directPlayObject := nil!

connectPollingProcess
	| pollingBlock |
	pollingBlock := [
		[true] whileTrue: [
			(Delay forMilliseconds: self class pollingRateInMilliseconds) wait. 
			self directPlayInactive ifTrue: [^self disconnectPollingProcess].
			[self receive] whileTrue: []]].
	pollingProcess := pollingBlock forkAt: Processor activePriority!

connectToExistingSession
	| list networkGroup message |
	list := directPlayObject networkSessions.
	list isEmpty ifTrue: [
		(self booleanPrompt: 'There are no existing sessions. Start a new one?')
			ifTrue: [^self connectToNewSession]
			ifFalse: [^self close]].
	networkGroup := self 
		selectFromList: list 
		title: 'Choose the network group to connect to.'.
	networkGroup isNil ifTrue: [^self]. "user cancelled"
	directPlayObject chooseSession: (list indexOf: networkGroup).
	self logonNetwork: #logonAsClientNamed:and:.
	self directPlayInactive ifTrue: [^self].
	self send: #(directPlayHello:).
	[self receive] whileFalse: [].!

connectToNetwork
	| pollingBlock |
	self openDirectPlay.
	self directPlayInactive ifTrue: [^self].
	self connectToProvider.
	self directPlayInactive ifTrue: [^self].
	(self booleanPrompt: 'Start a new network group?')
		ifTrue: [self connectToNewSession]
		ifFalse: [self connectToExistingSession].
	self directPlayInactive ifTrue: [^self].
	self connectPollingProcess!

connectToNewSession
	self logonNetwork: #logonAsServerNamed:and:.
	self send: #(directPlayHello:)!

connectToProvider
	| list networkProvider networkGroup logonResult |
	list := directPlayObject networkProviders.
	networkProvider := self selectFromList: list title: 'Choose a network provider.'.
	networkProvider isNil ifTrue: [^self close]. "user cancelled"
	directPlayObject chooseNetworkProvider: (list indexOf: networkProvider).!

directPlayActive
	^directPlayObject notNil!

directPlayGarbage: strings
	Transcript cr; show: 'Garbage ignored: ', strings printString!

directPlayHello: strings
	self isServer ifTrue: [self informDirectPlayOfSelection]. !

directPlayInactive
	^directPlayObject isNil!

directPlaySelect: strings
	self privateSelectClass: (strings at: 2) selector: strings last.!

disconnectFromNetwork
	ActiveNetEnabledBrowser := nil.
	self disconnectPollingProcess.
	self closeDirectPlay!

disconnectPollingProcess
	pollingProcess isNil ifTrue: [^self].
	pollingProcess terminate.
	pollingProcess := nil!

error
	^self errorString notEmpty!

errorString
	^directPlayObject errorString!

guidString
	"The following global unique identifier was obtained by running 
	c:\msdev\bin\uuidgen.exe in Microsoft Windows 95."
	^'{d4ba6b60-00d7-11d0-88d2-00a024a57dd9}' asDBString
!

informDirectPlayOfSelection
	| className methodName stream |
	className := self selectedClasses size = 1 
		ifTrue: [self selectedClasses first name] 
		ifFalse: [''].
	self showingInstanceMethods ifFalse: [
		className := className, ' class'].
	methodName := self selectedMethodNames size = 1 
		ifTrue: [self selectedMethodNames first]
		 ifFalse: [''].
	self send: (Array with: #directPlaySelect: 
		with: className with: methodName)!

isServer
	^directPlayObject logonType == #server!

logonNetwork: logonType
	| identification separatorIndex friendlyName formalName |
	identification := self stringPrompt: 'Formal name, friendly name'.
	separatorIndex := identification indexOf: $,.
	separatorIndex = 0 
		ifTrue: [formalName := friendlyName := identification]
		ifFalse: [
			formalName := identification copyFrom: 1 to: separatorIndex - 1. 
			friendlyName := identification copyFrom: separatorIndex + 1 to: identification size].
	directPlayObject 
		perform: logonType 
		with: formalName trimBlanks 
		with: friendlyName trimBlanks.
	self onErrorCloseDirectPlayWith: 'Failed to logon'!

messagePrompt: title
	^System message: title!

onErrorCloseDirectPlayWith: message
	self error ifTrue: [
		self messagePrompt: message, ': ', self errorString.
		self disconnectFromNetwork]!

open
	ActiveNetEnabledBrowser notNil ifTrue: [
		^ActiveNetEnabledBrowser bringToFront].
	ActiveNetEnabledBrowser := self.
	self connectToNetwork.
	self directPlayInactive ifTrue: [
		self messagePrompt: 'Direct play did NOT start up. Browser will not open.'.
		^self disconnectFromNetwork]. 
	super open.!

openDirectPlay
	directPlayObject := DirectPlay new open: self guidString.
	self onErrorCloseDirectPlayWith: 'Failed to connect to network'!

privateSelectClass: classString selector: selectorString
	"If it exists, find the method in the supplied class and selects it."

	self 
		withClassName: classString selectorName: selectorString 
		ifError:  [^self]
		otherwise: [:class :method :isInstanceMethod |
			self 
				classesSelected: (Array with: class);
				applications: (Array with: method application);
				selectedCategories: (class categoriesFor: selectorString asSymbol);
				showingPublicClasses: method isPublic;
				showingInstanceMethods: isInstanceMethod;
				selectMethodNamed: method selector.
			self selectClass: class.
			super changedSelectedApplications.
			super changedSelectedMethods].!

receive
	"Poll for a message and decode it when one arrives; i.e., process it."
	| strings selector |
	strings := directPlayObject receiveStrings.
	strings isEmpty ifTrue: [^false].

	selector := strings first asSymbol.
	(self respondsTo: selector) ifFalse: [self directPlayGarbage: strings. ^false].
	self perform: selector with: strings.
	^true!

selectFromList: aCollection title: title
	^CwListPrompter new
		parent: self shell;
		messageString: title;
		selectedItems: #();
		items: aCollection;
		printBlock: nil;
		prompt!

send: aCollectionOfStrings
	"Send the data direct."
	directPlayObject sendStrings: aCollectionOfStrings!

stringPrompt: title
	^System prompt: title!

windowClose: w clientData: ignore1 callData: doitRec
	super windowClose: w clientData: ignore1 callData: doitRec.
	self disconnectFromNetwork!

withClassName: classString selectorName: selectorString 
ifError: errorBlock otherwise: classAndMethodBlock
	"See privateSelectMethod:inClass for example usage."

	| hasClassSuffix className class dictionaryClass method |
	
	hasClassSuffix := classString size > 5 
		and: [(classString copyFrom: classString size - 5 to: classString size) = ' class'].

	className := hasClassSuffix
		ifTrue: [classString copyFrom: 1 to: classString size - 6]
		ifFalse: [classString].

	class := Smalltalk at: className asSymbol 
		ifAbsent: [^errorBlock value].

	dictionaryClass := hasClassSuffix
		ifTrue: [class class]
		ifFalse: [class].

	method := dictionaryClass methodDictionary 
		at: selectorString asSymbol
		ifAbsent: [^errorBlock value].

	^classAndMethodBlock value: class value: method value: hasClassSuffix not

	! !

NetEnabledBrowser categoriesFor: #directPlayGarbage: are:((OrderedCollection new)
	add: 'message processing';
	yourself)!
NetEnabledBrowser categoriesFor: #close are:((OrderedCollection new)
	add: 'opening/closing';
	yourself)!
NetEnabledBrowser categoriesFor: #errorString are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #changedSelectedMethods are:((OrderedCollection new)
	add: 'overriding browser';
	yourself)!
NetEnabledBrowser categoriesFor: #onErrorCloseDirectPlayWith: are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #connectPollingProcess are:((OrderedCollection new)
	add: 'opening/closing';
	yourself)!
NetEnabledBrowser categoriesFor: #directPlayInactive are:((OrderedCollection new)
	add: 'network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #messagePrompt: are:((OrderedCollection new)
	add: 'prompters';
	yourself)!
NetEnabledBrowser categoriesFor: #isServer are:((OrderedCollection new)
	add: 'network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #connectToExistingSession are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #guidString are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #connectToProvider are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #logonNetwork: are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #privateSelectClass:selector: are:((OrderedCollection new)
	add: 'message generation';
	yourself)!
NetEnabledBrowser categoriesFor: #selectFromList:title: are:((OrderedCollection new)
	add: 'prompters';
	yourself)!
NetEnabledBrowser categoriesFor: #closeDirectPlay are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #booleanPrompt: are:((OrderedCollection new)
	add: 'prompters';
	yourself)!
NetEnabledBrowser categoriesFor: #withClassName:selectorName:ifError:otherwise: are:((OrderedCollection new)
	add: 'message generation';
	yourself)!
NetEnabledBrowser categoriesFor: #directPlayHello: are:((OrderedCollection new)
	add: 'message processing';
	yourself)!
NetEnabledBrowser categoriesFor: #connectToNetwork are:((OrderedCollection new)
	add: 'network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #stringPrompt: are:((OrderedCollection new)
	add: 'prompters';
	yourself)!
NetEnabledBrowser categoriesFor: #disconnectFromNetwork are:((OrderedCollection new)
	add: 'network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #send: are:((OrderedCollection new)
	add: 'network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #open are:((OrderedCollection new)
	add: 'opening/closing';
	yourself)!
NetEnabledBrowser categoriesFor: #disconnectPollingProcess are:((OrderedCollection new)
	add: 'opening/closing';
	yourself)!
NetEnabledBrowser categoriesFor: #informDirectPlayOfSelection are:((OrderedCollection new)
	add: 'message generation';
	yourself)!
NetEnabledBrowser categoriesFor: #windowClose:clientData:callData: are:((OrderedCollection new)
	add: 'opening/closing';
	yourself)!
NetEnabledBrowser categoriesFor: #directPlaySelect: are:((OrderedCollection new)
	add: 'message processing';
	yourself)!
NetEnabledBrowser categoriesFor: #error are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #connectToNewSession are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #openDirectPlay are:((OrderedCollection new)
	add: 'support for network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #receive are:((OrderedCollection new)
	add: 'network communication';
	yourself)!
NetEnabledBrowser categoriesFor: #directPlayActive are:((OrderedCollection new)
	add: 'network communication';
	yourself)!

DirectPlay initializeAfterLoad!
NetEnabledBrowser initializeAfterLoad!
NetEnablingPackage initializeAfterLoad!

NetEnablingPackage loaded!